package info.bukova.isspst.ui.workgroups;

import info.bukova.isspst.Constants;
import info.bukova.isspst.data.JobMapping;
import info.bukova.isspst.data.Member;
import info.bukova.isspst.data.Role;
import info.bukova.isspst.data.User;
import info.bukova.isspst.data.Workgroup;
import info.bukova.isspst.services.users.RoleService;
import info.bukova.isspst.services.users.UserService;
import info.bukova.isspst.services.workgroups.WorkgroupService;
import info.bukova.isspst.ui.FormViewModel;

import java.util.ArrayList;
import java.util.List;
import java.util.Set;

import org.zkoss.bind.annotation.BindingParam;
import org.zkoss.bind.annotation.Command;
import org.zkoss.bind.annotation.Init;
import org.zkoss.bind.annotation.NotifyChange;
import org.zkoss.zk.ui.event.DropEvent;
import org.zkoss.zk.ui.select.annotation.WireVariable;
import org.zkoss.zul.Listitem;

public class WorkgroupForm extends FormViewModel<Workgroup> {
	
	@WireVariable
	private UserService userService;
	@WireVariable
	private RoleService roleService;
	@WireVariable
	private WorkgroupService workgroupService;
	private List<Member> users;

	@Init(superclass = true)
	public void init() {
		users = new ArrayList<Member>();
		users.addAll(userService.getAll());
	}
	
	public List<Member> getUsers() {
		if (getDataBean().isCentre()) {
			List<Member> listWithWg = new ArrayList<Member>(users);
			
			List<Workgroup> wg = workgroupService.getWorkgroups();
			
			for (Workgroup w : wg) {
				if (!workgroupService.isMember(getDataBean(), w)) {
					listWithWg.add(w);
				}
			}
			
			return listWithWg;
		}
		
		return users;
	}
	
	public List<Role> getWorkgroupRoles() {
		if (getDataBean().isCentre()) {
			return roleService.getCentreRoles();
		} else {
			return roleService.getWorkgroupRoles();
		}
	}
	
	@NotifyChange({"workgroupRoles", "centre", "users"})
	public void setCentre(boolean centre) {
		getDataBean().setCentre(centre);
		getDataBean().getMembers().clear();
		
		if (centre) {
			List<Role> centreRoles = roleService.getCentreRoles();
			for (Role r : centreRoles) {
				if (!r.getAuthority().equals(Constants.ROLE_USER)) {
					List<User> users = userService.getUsersWithRole(r);
					for (User u : users) {
						workgroupService.addMember(getDataBean(), u, r);
					}
				}
			}
		}
	}
	
	public boolean getCentre() {
		return getDataBean().isCentre();
	}
	
	@Command
	@NotifyChange({"users", "dataBean"})
	public void addMember(@BindingParam("event") DropEvent event) {
		 Set<Listitem> selected = ((Listitem)event.getDragged()).getListbox().getSelectedItems();
		 
		 if (selected.isEmpty()) {
			 try {
				 Member u = ((Listitem)event.getDragged()).getValue();
				 moveUser(u, event);
			 } catch(ClassCastException e) {
				 JobMapping u = ((Listitem)event.getDragged()).getValue();
				 moveUser(u, event);
			 }
			 
			 
			 return;
		 }
		 
		 for (Listitem li : selected) {
			 try {
				 Member u = li.getValue();
				 moveUser(u, event);
			 } catch (ClassCastException e) {
				 JobMapping u = li.getValue();
				 moveUser(u, event);
			 }
		 }
	}
	
	private void moveUser(JobMapping u, DropEvent event) {
		String source = getSource(event);
		String target = getTarget(event);
		
		if (source.equals(target)) {
			return;
		}
		
		workgroupService.removeMember(getDataBean(), u);
		
		if (!target.equals("users")) {
			moveUser(u.getMember(), event);
		}
	}

	private void moveUser(Member u, DropEvent event) {
		String source = getSource(event);
		String target = getTarget(event);
		
		if (source.equals(target)) {
			return;
		}
		
		workgroupService.addMember(getDataBean(), u, roleService.getRoleByAuthority(target));
		
		if (!u.isPerson()) {
			users.remove(u);
		}
	}
	
	private String getSource(DropEvent event) {
		return ((Listitem)event.getDragged()).getListbox().getId();
	}
	
	private String getTarget(DropEvent event) {
		String target;
		
		if (event.getTarget() instanceof Listitem) {
			target = ((Listitem)event.getTarget()).getListbox().getId();
		} else {
			target = event.getTarget().getId();
		}
		
		return target;
	}
	
}
